/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.registry.overlay;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.clientserverapi.configuration.ContainerConfiguration;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.ContainerConfigurationListener;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration;
import org.ow2.petals.microkernel.api.configuration.exception.ConfigurationException;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerAlreadyExistException;
import org.ow2.petals.microkernel.api.configuration.exception.ContainerDoesNotExistException;
import org.ow2.petals.microkernel.api.configuration.exception.DomainUnknownException;
import org.ow2.petals.microkernel.api.configuration.exception.InvalidContainerConfigurationProvidedException;
import org.ow2.petals.microkernel.api.configuration.exception.MissingServerPropertyException;
import org.ow2.petals.topology.generated.Topology;

/**
 * An implementation of {@link ConfigurationService} used to unit test {@link DynamicTopologyListener}.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DynamicTopologyListenerTestCfgSvc implements ConfigurationService {

    private final Map<String, ContainerConfiguration> containerConfig = new HashMap<String, ContainerConfiguration>();

    private final boolean waitOnAdd;

    private final ContainerConfiguration currentContainerConfig;

    public DynamicTopologyListenerTestCfgSvc(final boolean waitOnAdd,
            final ContainerConfiguration currentContainerConfig,
            final ContainerConfiguration[] remoteContainerConfigs) {
        this.waitOnAdd = waitOnAdd;
        this.currentContainerConfig = currentContainerConfig;
        if (remoteContainerConfigs != null) {
            for (final ContainerConfiguration remoteContainerConfig : remoteContainerConfigs) {
                this.containerConfig.put(remoteContainerConfig.getName(), remoteContainerConfig);
            }
        }
    }

    @Override
    public ContainerConfiguration getContainerConfiguration() {
        return this.currentContainerConfig;
    }

    @Override
    public Map<String, String> getAllServerProperties() {
        return null;
    }

    @Override
    public Properties getServerProperties() {
        return null;
    }

    @Override
    public Set<ContainerConfiguration> getContainersConfiguration() {
        return new HashSet<>(this.containerConfig.values());
    }

    @Override
    public DomainConfiguration getDomainConfiguration() {
        return null;
    }

    @Override
    public Topology getLocalTopology() {
        return null;
    }

    @Override
    public ContainerConfiguration getContainerConfiguration(final String containerName) {
        return null;
    }

    @Override
    public void addContainerConfiguration(final ContainerConfiguration addedContainer)
            throws InvalidContainerConfigurationProvidedException, ContainerAlreadyExistException,
            DomainUnknownException {
        this.containerConfig.put(addedContainer.getName(), addedContainer);
        if (this.waitOnAdd) {
            synchronized (this) {
                this.notifyAll();
            }
        }
    }

    @Override
    public void removeContainerConfiguration(final ContainerConfiguration removedContainer)
            throws ContainerDoesNotExistException, InvalidContainerConfigurationProvidedException {
        this.containerConfig.remove(removedContainer.getName());
        if (!this.waitOnAdd) {
            synchronized (this) {
                this.notifyAll();
            }
        }
    }

    @Override
    public void loadConfiguration(final Properties serverLocalProperties, final Topology topology)
            throws MissingServerPropertyException, ConfigurationException {
        // NOP
    }

    @Override
    public void updateTopology(final Topology topology) throws ConfigurationException {
        // NOP
    }

    @Override
    public void reconfigureDomain(final DomainConfiguration newDomainConfiguration) {
        // NOP
    }

    @Override
    public boolean isSecurityTopologyPassphraseOk(final String securityPassPhrase) {
        return false;
    }

    @Override
    public void registerContainerConfigurationListener(ContainerConfigurationListener listener) {
        // NOP
    }

    @Override
    public void deregisterContainerConfigurationListener(ContainerConfigurationListener listener) {
        // NOP
    }

    @Override
    public void persistTopology(Topology topology) throws ConfigurationException {
        // NOP
    }

}
